package jadex.bdi.examples.disastermanagement;

import jadex.application.space.envsupport.environment.AbstractTask;
import jadex.application.space.envsupport.environment.IEnvironmentSpace;
import jadex.application.space.envsupport.environment.ISpaceObject;
import jadex.commons.service.clock.IClockService;

/**
 *  Deliver a patient at the hospital.
 */
public class DeliverPatientTask extends AbstractTask
{
	//-------- constants --------
	
	/** The task name. */
	public static final String	PROPERTY_TYPENAME = "deliver_patient";
	
	/** The treated property (of the space object). */
	public static final String	PROPERTY_TREATED = "treated";

	/** The patient property (of the space object). */
	public static final String	PROPERTY_PATIENT = "patient";

	//-------- IObjectTask methods --------
	
	/**
	 *  Executes the task.
	 *  @param space	The environment in which the task is executing.
	 *  @param obj	The object that is executing the task.
	 *  @param progress	The time that has passed according to the environment executor.
	 */
	public void execute(IEnvironmentSpace space, ISpaceObject obj, long progress, IClockService clock)
	{
		// Update ambulance object based on time progress.
		double	treated	= ((Number)obj.getProperty(PROPERTY_TREATED)).doubleValue();
		treated	+= progress*0.0005;	// 1 patient per 2 seconds.
		if(treated>1)
		{
			obj.setProperty(PROPERTY_PATIENT, Boolean.FALSE);
			obj.setProperty(PROPERTY_TREATED, new Double(0));
			obj.setProperty("state", "idle");
			setFinished(space, obj, true);
		}
		else
		{
			obj.setProperty(PROPERTY_TREATED, new Double(treated));			
		}
	}
}
